#!/bin/bash
#
# This file was automatically generated by s2e-env at 2019-01-25 12:50:49.815180
#
# This bootstrap file is used to control the execution of the Linux target
# program in S2E.
#
# When you run launch-s2e.sh, the guest VM calls s2eget to fetch and execute this
# bootstrap script. This bootstrap script and the S2E config file determine how
# the target program is analyzed.
#

set -x

# To save the hassle of rebuilding guest images every time you update S2E's guest tools,
# the first thing that we do is get the latest versions of the guest tools.
function update_guest_tools {
    local GUEST_TOOLS
    local OUR_S2EGET

    GUEST_TOOLS="$COMMON_TOOLS $(target_tools)"
    OUR_S2EGET=${S2EGET}



    for TOOL in ${GUEST_TOOLS}; do
        ${OUR_S2EGET} guest-tools/${TOOL}
        chmod +x ${TOOL}
    done
}

function prepare_target {
    # Make sure that the target is executable
    chmod +x "$1"
}

# This prepares the symbolic file inputs.
# This function takes as input an optional seed file name.
# If the seed file is present, the commands makes the seed symbolic.
# Otherwise, it creates an empty file.
#
# Symbolic files must be stored in a ram disk, as only memory (and cpu)
# is capable of holding symbolic data.
#
# This function prints the path to the symbolic file on stdout.
function prepare_inputs {
    local SEED_FILE
    local SYMB_FILE

    # This can be empty if there are no seed files
    SEED_FILE="$1"

    # Check whether the target has custom handling
    # of seed files.
    if [ $(make_seeds_symbolic) -eq 0 ]; then
        echo ${SEED_FILE}
        return
    fi


    SYMB_FILE="/tmp/input"


    if [ "x$SEED_FILE" = "x" ]; then
        # Create a symbolic file of size 256 bytes.
        # Note: you can customize this commands according to your needs.
        # You could, e.g., use non-zero input, different sizes, etc.

        truncate -s 256 ${SYMB_FILE}


        if [ $? -ne 0 ]; then
            ${S2ECMD} kill 1 "Failed to create symbolic file"
            exit 1
        fi
    else
        ${S2EGET} ${SEED_FILE} >/dev/null
        if [ ! -f ${SEED_FILE} ]; then
           ${S2ECMD} kill 1 "Could not fetch seed file ${SEED_FILE} from the host"
        fi


        cp ${SEED_FILE} ${SYMB_FILE}


        ${S2EGET} ${SEED_FILE}.symranges >/dev/null
    fi

    # Make the file symbolic
    if [ -f "${SEED_FILE}.symranges" ]; then
       export S2E_SYMFILE_RANGES="${SEED_FILE}.symranges"
    fi


    # The symbolic file will be split into symbolic variables of up to 4k bytes each.
    ${S2ECMD} symbfile 4096 ${SYMB_FILE} >/dev/null

    echo ${SYMB_FILE}
}


# This function executes the target program given in arguments.
#
# There are two versions of this function:
#    - without seed support
#    - with seed support (-s argument when creating projects with s2e_env)
function execute {
    local TARGET
    local SEED_FILE
    local SYMB_FILE

    TARGET=$1

    prepare_target "${TARGET}"


    SEED_FILE="$2"
    SYMB_FILE="$(prepare_inputs $SEED_FILE)"

    execute_target "${TARGET}" "${SYMB_FILE}"

}


# # This function executes the target program given in arguments.
# #
# # There are two versions of this function:
# #    - without seed support
# #    - with seed support (-s argument when creating projects with s2e_env)
# function execute {
#     local TARGET
#     local SEED_FILE
#     local SYMB_FILE

#     TARGET=$1

#     prepare_target "${TARGET}"


#     # In seed mode, state 0 runs in an infinite loop trying to fetch and
#     # schedule new seeds. It works in conjunction with the SeedSearcher plugin.
#     # The plugin schedules state 0 only when seeds are available.

#     # Enable seeds and wait until a seed file is available. If you are not
#     # using seeds then this loop will not affect symbolic execution - it will
#     # simply never be scheduled.
#     ${S2ECMD} seedsearcher_enable
#     while true; do
#         SEED_FILE=$(${S2ECMD} get_seed_file)

#         if [ $? -eq 1 ]; then
#             # Avoid flooding the log with messages if we are the only runnable
#             # state in the S2E instance
#             sleep 1
#             continue
#         fi

#         break
#     done

#     if [ -n "${SEED_FILE}" ]; then
#         SYMB_FILE="$(prepare_inputs ${SEED_FILE})"
#         execute_target "${TARGET}" "${SYMB_FILE}"
#     else
#         # If there are no seeds available, execute the seedless instance.
#         # The SeedSearcher only schedules the seedless instance once.
#         #
#         echo "Starting seedless execution"

#         # NOTE: If you do not want to use seedless execution, comment out
#         # the following line.
#         execute_target "${TARGET}"
#     fi


# }

###############################################################################
# This section contains target-specific code

# CGC has a different way of making seeds symbolic,
# override default behavior.
function make_seeds_symbolic {
    echo 0
}

function execute_target {
    local TARGET
    local SEED_FILE

    TARGET="$1"
    SEED_FILE="$2"

    if [ "x${SEED_FILE}" = "x" ]; then
        ./${TARGET} > /tmp/out 2>&1
    else
        # Make the seed file concolic and submit it to the cb-test application.
        # Note: CGC files don't need to be in a ram disk, as they will be made
        # symbolic at the syscall level. See DecreeMonitor for details.
        ${S2EGET} "${SEED_FILE}"
        ./cgccmd concolic on

        chmod +x ${SEED_FILE}
        cb-test --directory $(pwd) --xml ${SEED_FILE} --cb ${TARGET} --should_core --timeout 3600 2>&1
    fi
}

function target_init {
    # Patch cb-test so that it works without core dumps
    sudo sed -i 's/resource.RLIM_INFINITY/0/g' /usr/bin/cb-test

    # Some binaries have strange headers, allow them here
    echo 1 | sudo tee /proc/sys/cgc/relaxed_headers

    # Start the DecreeMonitor kernel module
    sudo modprobe s2e
}

function target_tools {
    echo "cgccmd"
}

S2ECMD=./s2ecmd
S2EGET=./s2eget
S2EPUT=./s2eput
COMMON_TOOLS="s2ecmd s2eget s2eput"

###############################################################################


update_guest_tools



# Don't print crashes in the syslog. This prevents unnecessary forking in the
# kernel
sudo sysctl -w debug.exception-trace=0

# Prevent core dumps from being created. This prevents unnecessary forking in
# the kernel
ulimit -c 0

# Ensure that /tmp is mounted in memory (if you built the image using s2e-env
# then this should already be the case. But better to be safe than sorry!)
if ! mount | grep "/tmp type tmpfs"; then
    sudo mount -t tmpfs -osize=10m tmpfs /tmp
fi

# Need to disable swap, otherwise there will be forced concretization if the
# system swaps out symbolic data to disk.
sudo swapoff -a



target_init

# Download the target file to analyze
${S2EGET} "target"



# Run the analysis
execute "./target" "crashing_input.xml"
# execute_target "./CROMU_00001" "crashing_input"

# Kill states before exiting
${S2ECMD} kill $? "Target execution terminated"
